@@ -16,6 +16,10 @@ from tensorkube.services.error import CLIVersionError
 from tensorkube.services.s3_service import list_s3_buckets
 
 
+def get_session_region():
+    return boto3.session.Session().region_name
+
+
 def get_eks_client():
     return boto3.client("eks")
 
