@@ -12,7 +12,7 @@ from tensorkube.helpers import create_mountpoint_driver_role_with_policy, saniti
 from tensorkube.migration_service.migration_manager.migration_service import set_current_cli_version_to_cluster, \
     migrate_tensorkube
 from tensorkube.services.aws_service import get_aws_account_id, get_credentials, get_bucket_name, get_aws_user_arn, \
-    get_aws_user_principal_arn
+    get_aws_user_principal_arn, get_session_region
 from tensorkube.services.cloudformation_service import delete_cloudformation_stack, delete_launch_templates, \
     cloudformation, get_validation_records_from_events, get_validation_records_with_wait
 from tensorkube.services.dataset_service import upload_tensorkube_dataset, delete_tensorkube_dataset
@@ -147,6 +147,22 @@ def configure():
         click.echo("You need to login to Tensorfuse to configure Tensorkube.")
         click.echo("Please login using the command: " + click.style("tensorkube login", fg='cyan'))
         return
+    confirmation_txt = click.style("Confirmation: Tensorfuse sets up resources such as a Kubernetes cluster, load balancer, and cluster autoscaler to enable serverless GPUs and features like job queues. The estimated base cost for this setup is approximately ") + click.style("$300 per month", bold=True) + click.style(", billed by AWS. Please select \"Yes\" if you wish to proceed.")
+    confirmation = click.confirm(confirmation_txt, default=False)
+    if not confirmation:
+        end_time = time.time() * 1000
+        track_event(Events.CONFIGURE_END.value,
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "confirmation_declined"})    
+        return
+    session_region = get_session_region()
+    if session_region != REGION:
+        end_time = time.time() * 1000
+        track_event(Events.CONFIGURE_END.value,
+                {"start_time": start_time, "end_time": end_time, "duration": end_time - start_time, "status": "region_not_supported", "region": session_region})    
+        click.echo(click.style( f"Error: The current region is {session_region}. Tensorfuse currently supports only {REGION} region. Either switch your region to {REGION} or reach out to ", bold=True,
+                        fg=CliColors.ERROR.value) + click.style( "founders@tensorfuse.io", italic=True, bold=True, fg=CliColors.ERROR.value) + click.style(" for support.", bold=True, fg=CliColors.ERROR.value))
+        return
+    
     if not check_and_install_cli_tools():
         return
     # TODO!: add helm annotations
@@ -249,9 +265,31 @@ def cluster():
 
 @tensorkube.command()
 def teardown():
+    """
+    HIGHLY DESTRUCTIVE ACTION!!!! Don't run this command if you don't intend to delete all the resources that you have created and configured using the CLI.
+
+    <Warning>
+    This action will remove everything you configured using tensorkube.
+    </Warning>
+    :return:
+    """
     start_time = time.time() * 1000
     track_event(Events.TEARDOWN_START.value, {"start_time": start_time})
 
+    # ask for the secret phrase before tearing down and inform the user this means that you are destroying all the resources
+    confirmation = click.confirm(click.style("Are you sure you want to tear down all resources? This will destroy all the resources that you have created and configured using Tensorfuse.", bold=True, fg=CliColors.ERROR.value), default=False)
+    if not confirmation:
+        click.echo("Tear down aborted.")
+        return
+    else:
+        # ask for  a secret phrase if given secret phrase is not correct then abort the teardown
+        secret_phrase = click.prompt(click.style("Please enter the secret phrase to confirm the teardown. Contact the founders for the phrase.", bold=True,fg=CliColors.ERROR.value), hide_input=True)
+        if secret_phrase == 'tensorsamagam':
+            click.echo("Tear down confirmed.")
+        else:
+            click.echo("Secret phrase is incorrect. Tear down aborted.")
+            return
+
     click.echo("Tearing down all resources...")
 
     # TODO?: add logic to delete any other resources
@@ -391,7 +429,7 @@ def teardown():
 
 @tensorkube.command()
 @click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for the service.')
-@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S','A100','H100'], case_sensitive=False),
               help='Type of GPU.')
 @click.option('--cpu', type=float, default=DEFAULT_CPU, help='Number of CPU millicores. 1000 = 1 CPU')
 @click.option('--memory', type=float, default=DEFAULT_MEMORY, help='Amount of RAM in megabytes.')
@@ -405,7 +443,8 @@ def teardown():
 @click.option('--config-file', type=str,
               help='Path to the config.yaml file. you can use this file instead of cli flags. Keys in config.yaml take precedence than cli flags.')
 @click.option('--domain-name', type=str, default=None, help='Domain to use for the deployment.')
-def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_actions, secret, efs, concurrency, config_file, domain_name):
+@click.option('--port', type=int, default=80, help='Port to expose the service on.')
+def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_actions, secret, efs, concurrency, config_file, domain_name, port):
     """
     Deploy your containerized application on Tensorkube. This command requires
     a dockerfile to be present in the current directory.
@@ -424,7 +463,7 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
         'memory', memory).set_if_not_exist('min_scale', min_scale).set_if_not_exist('max_scale',
                                                                                     max_scale).set_if_not_exist('env',
                                                                                                                 env).set_if_not_exist(
-        'github_actions', github_actions).set_if_not_exist('secret', secret).set_if_not_exist('domain', domain_name).set_if_not_exist('efs', efs).set_if_not_exist('concurrency', concurrency)
+        'github_actions', github_actions).set_if_not_exist('secret', secret).set_if_not_exist('domain', domain_name).set_if_not_exist('efs', efs).set_if_not_exist('concurrency', concurrency).set_if_not_exist('port', port)
 
     # flags validation
     if config.get('gpus') > 0:
@@ -470,7 +509,7 @@ def deploy(gpus, gpu_type, cpu, memory, min_scale, max_scale, env, github_action
                            gpu_type=config.get('gpu_type'), cpu=config.get('cpu'), memory=config.get('memory'),
                            min_scale=config.get('min_scale'), max_scale=config.get('max_scale'),concurrency= config.get('concurrency'), pod_status=pod_status,
                            cwd=cwd, sanitised_project_name=sanitised_project_name, image_url=image_url, secrets=depl_secrets,
-                           enable_efs=enable_efs, domain=config.get('domain'),github_actions=config.get('github_actions'), readiness = config.get('readiness'))
+                           enable_efs=enable_efs, domain=config.get('domain'),github_actions=config.get('github_actions'), readiness = config.get('readiness'), port = config.get('port'))
 
     end_time = time.time() * 1000
     track_event(Events.DEPLOY_END.value,
@@ -673,7 +712,7 @@ def login():
 
 @secret.command()
 @click.option('--env', default=DEFAULT_NAMESPACE, type=str, help='Environment to create the secret in.')
-@click.option('--force', is_flag=True, help='Update the secret if one with the same neame already exists.')
+@click.option('--force', is_flag=True, help='Update the secret if one with the same name already exists.')
 @click.argument('secret_name', type=str)
 @click.argument('key_value_pairs', nargs=-1)
 def create(secret_name, key_value_pairs, env, force):
@@ -824,7 +863,7 @@ def train():
 
 @train.command()
 @click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for the service.')
-@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S', 'A100', 'H100'], case_sensitive=False),
               help='Type of GPU.')
 @click.option('--env', default=DEFAULT_ENV, type=str, help='Environment to deploy the service to.')
 @click.option('--job-id', default=None, type=str, help='Unique job id for the training job.')
@@ -943,7 +982,7 @@ def job():
 @job.command()
 @click.option('--name', required=True, type=str, help='Name of the job to deploy.')
 @click.option('--gpus', default=DEFAULT_GPUS, help='Number of GPUs needed for an instance of the job.')
-@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S'], case_sensitive=False),
+@click.option('--gpu-type', type=click.Choice(['V100', 'A10G', 'T4', 'L4', 'L40S', 'A100', 'H100'], case_sensitive=False),
               help='Type of GPU.')
 @click.option('--cpu', type=float, default=DEFAULT_CPU, help='Number of CPU millicores. 1000 = 1 CPU')
 @click.option('--memory', type=float, default=DEFAULT_MEMORY, help='Amount of RAM in megabytes.')
