@@ -133,7 +133,9 @@ def build_app(env, sanitised_project_name, image_tag, upload_to_nfs: bool = Fals
     
     apply_k8s_buildkit_config(sanitised_project_name=sanitised_project_name, image_tag=image_tag, env_name=env, image_url=image_url, upload_to_nfs=upload_to_nfs,convert_to_nydus_image=convert_to_nydus_image)
     build_job_pod_name = get_pod_name_corresponing_to_job(job_name=build_job_name, namespace=env_namespace)
-
+    if build_job_pod_name is None:
+        click.echo("Build Pod not found")
+        return PodStatus.FAILED.value
     # TODO: stream multiple lines instead of one by one
     start_streaming_pod(pod_name=build_job_pod_name, namespace=env_namespace, status=PodStatus.SUCCEEDED)
 
@@ -158,7 +160,7 @@ def build_app(env, sanitised_project_name, image_tag, upload_to_nfs: bool = Fals
     return pod_status
 
 def deploy_knative_service(env: str, gpus: int, gpu_type: str, cpu: float, memory: float, min_scale: int, max_scale: int, concurrency: int,
-                           pod_status: str, cwd: str, sanitised_project_name: str, image_url: str, secrets: list, domain: Optional[str],
+                           pod_status: str, cwd: str, sanitised_project_name: str, image_url: str, secrets: list, port: int, domain: Optional[str],
                            enable_efs: bool = False, github_actions: bool = DEFAULT_GITHUB_ACTIONS, readiness: Optional[Dict] = None):
     env_namespace = DEFAULT_NAMESPACE if not env else env
 
@@ -173,7 +175,7 @@ def deploy_knative_service(env: str, gpus: int, gpu_type: str, cpu: float, memor
                                             sanitised_project_name=sanitised_project_name, image_tag=image_url,
                                             workdir=workdir, command=command, gpus=gpus, gpu_type=gpu_type, cpu=cpu,
                                             memory=memory, min_scale=min_scale, max_scale=max_scale, env=env,
-                                            secrets=secrets, enable_efs=enable_efs, domain=domain, concurrency=concurrency, readiness = readiness)
+                                            secrets=secrets, enable_efs=enable_efs, domain=domain, concurrency=concurrency, readiness = readiness, port = port)
         if domain is None:
             virtual_service_yaml_file_path = resource_filename('tensorkube',
                                                                'configurations/build_configs/virtual_service.yaml')
