@@ -697,22 +697,23 @@ def get_image_tags_to_retain(sanitised_project_name: str, service_name: str, nam
 
 
 def get_pod_name_corresponing_to_job(job_name: str, namespace: str = DEFAULT_NAMESPACE,
-                                     context_name: Optional[str] = None) -> Optional[str]:
+                                     context_name: Optional[str] = None, max_retries = 5) -> Optional[str]:
     if not context_name:
         context_name = get_tensorkube_cluster_context_name()
         if not context_name:
             return None
     k8s_api_client = config.new_client_from_config(context=context_name)
     v1 = client.CoreV1Api(k8s_api_client)
-    pods = v1.list_namespaced_pod(namespace=namespace)
-
-    for pod in pods.items:
-        if not pod.metadata.owner_references:
-            continue
-        for owner in pod.metadata.owner_references:
-            if owner.name == job_name:
-                return pod.metadata.name
-
+    for attempt in range(max_retries):
+        pods = v1.list_namespaced_pod(namespace=namespace)
+        for pod in pods.items:
+            if not pod.metadata.owner_references:
+                continue
+            for owner in pod.metadata.owner_references:
+                if owner.name == job_name:
+                    return pod.metadata.name
+        backoff_delay = 2 ** attempt
+        time.sleep(backoff_delay)
     return None
 
 
