@@ -8,8 +8,8 @@ from tensorkube.constants import get_cluster_name
 from tensorkube.services.k8s_service import get_tensorkube_cluster_context_name
 
 
-def apply_ec2nodeclass(context_name: Optional[str] = None):
-    file_name = resource_filename('tensorkube', 'configurations/karpenter_ec2nodeclass.yaml')
+def apply_ec2nodeclass(filename: str, ec2nodeclass_name: str, context_name: Optional[str] = None):
+    file_name = resource_filename('tensorkube', f'configurations/{filename}')
     with open(file_name, 'r') as file:
         yaml_doc = yaml.safe_load(file)
 
@@ -25,7 +25,6 @@ def apply_ec2nodeclass(context_name: Optional[str] = None):
     version = "v1beta1"
     namespace = "default"
     plural = "ec2nodeclasses"
-    name = "default"
 
     yaml_doc['spec']['role'] = f"KarpenterNodeRole-{get_cluster_name()}"
     yaml_doc['spec']['subnetSelectorTerms'][0]['tags']['karpenter.sh/discovery'] = get_cluster_name()
@@ -34,13 +33,13 @@ def apply_ec2nodeclass(context_name: Optional[str] = None):
     try:
         # Check if the resource already exists
         existing_resource = api_instance.get_cluster_custom_object(group=group, version=version, plural=plural,
-                                                                   name=name)
-        print(f"Resource {name} already exists. Skipping creation.")
+                                                                   name=ec2nodeclass_name)
+        print(f"Resource {ec2nodeclass_name} already exists. Skipping creation.")
     except client.exceptions.ApiException as e:
         if e.status == 404:
             # Resource does not exist, proceed to create
             api_instance.create_cluster_custom_object(group=group, version=version, plural=plural, body=yaml_doc)
-            print(f"Resource {name} created successfully.")
+            print(f"Resource {ec2nodeclass_name} created successfully.")
         else:
             print(f"An error occurred: {e}")
             raise e
@@ -84,8 +83,8 @@ def upgrade_karpenter_ec2nodeclass(context_name: Optional[str] = None):
         raise e
 
 
-def apply_nodepools(context_name: Optional[str] = None):
-    file_name = resource_filename('tensorkube', 'configurations/karpenter_nodepool.yaml')
+def apply_nodepools(filename: str, nodepool_name: str, context_name: Optional[str] = None):
+    file_name = resource_filename('tensorkube', f'configurations/{filename}')
     with open(file_name, 'r') as file:
         yaml_doc = yaml.safe_load(file)
 
@@ -101,17 +100,17 @@ def apply_nodepools(context_name: Optional[str] = None):
     version = "v1beta1"
     namespace = "default"
     plural = "nodepools"
-    name = "default"
+
     try:
         # Check if the resource already exists
         existing_resource = api_instance.get_cluster_custom_object(group=group, version=version, plural=plural,
-                                                                   name=name)
-        print(f"Resource {name} already exists. Skipping creation.")
+                                                                   name=nodepool_name)
+        print(f"Resource {nodepool_name} already exists. Skipping creation.")
     except client.exceptions.ApiException as e:
         if e.status == 404:
             # Resource does not exist, proceed to create
             api_instance.create_cluster_custom_object(group=group, version=version, plural=plural, body=yaml_doc)
-            print(f"Resource {name} created successfully.")
+            print(f"Resource {nodepool_name} created successfully.")
         else:
             print(f"An error occurred: {e}")
             raise e
@@ -154,8 +153,8 @@ def upgrade_karpenter_nodepools(context_name: Optional[str] = None):
 
 
 def apply_karpenter_configuration():
-    apply_ec2nodeclass()
-    apply_nodepools()
+    apply_ec2nodeclass("karpenter_ec2nodeclass.yaml", "default")
+    apply_nodepools("karpenter_nodepool.yaml", "default")
 
 
 def delete_ec2nodeclasses(context_name: Optional[str] = None):
