@@ -28,6 +28,10 @@ def get_instance_family_from_gpu_type(gpu_type):
         return 'g6'
     elif gpu_type == 'l40s':
         return 'g6e'
+    elif gpu_type == 'a100':
+        return 'p4d'
+    elif gpu_type == 'h100':
+        return 'p5'
     else:
         raise ValueError(f"Unsupported GPU type: {gpu_type}")
 
@@ -44,6 +48,10 @@ def get_gpu_type_from_instance_family(instance_family):
         return 'l4'
     elif instance_family == 'g6e':
         return 'l40s'
+    elif instance_family == 'p4d':
+        return 'a100'
+    elif instance_family == 'p5':
+        return 'h100'
     else:
         raise ValueError(f"Unsupported instance family: {instance_family}")
 
@@ -237,9 +245,9 @@ def apply_tls_virtual_service(service_name: str, domain: str, yaml_file_path: st
 
 
 def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_project_name: str,
-                                      image_tag: str, workdir: str, command: str, gpus: int, gpu_type: str,
+                                      image_tag: str, workdir: str, command: str, gpus: int, gpu_type: str, port: int,
                                       domain: Optional[str], env: Optional[str] = None, cpu: float = 100,
-                                      memory: int = 200, min_scale: int = 0, max_scale: int = 3,
+                                      memory: int = 200, min_scale: int = 0, max_scale: int = 3, 
                                       context_name: Optional[str] = None, secrets: List[str] = [], enable_efs: bool = False, concurrency: int = 100, readiness: Optional[Dict] = None):
 
     # Load kube config
@@ -402,6 +410,12 @@ def apply_knative_service(service_name: str, yaml_file_path: str, sanitised_proj
 
     if readiness:
         yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe'] = readiness
+    
+    # set container port
+    yaml_dict['spec']['template']['spec']['containers'][0]['ports'] = [{'containerPort': port}]
+    if readiness is None:
+        if 'httpGet' in yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe']:
+            yaml_dict['spec']['template']['spec']['containers'][0]['readinessProbe']['httpGet']['port'] = port
 
 
     # enable http redirects if domain is available
