@@ -43,7 +43,7 @@ def check_and_install_kubectl():
 
         # Verify kubectl installation
         try:
-            subprocess.run(["kubectl", "version", "--client"], shell=True, check=True)
+            subprocess.run("kubectl version --client", shell=True, check=True)
             print("kubectl installed successfully.")
         except subprocess.CalledProcessError as e:
             print("kubectl installation failed. Please install kubectl manually.")
@@ -131,8 +131,9 @@ def install_kubectl(system, architecture):
     try:
         # Download kubectl binary for the specified architecture
         click.echo(f"Downloading kubectl binary for {architecture} architecture...")
+        cmd = f"curl -LO https://dl.k8s.io/release/v{LOCKED_KUBECTL_VERSION}/bin/{system}/{architecture}/kubectl"
         subprocess.run(
-            f"curl -LO https://dl.k8s.io/release/$(curl -L -s https://dl.k8s.io/release/stable.txt)/bin/{system}/{architecture}/kubectl",
+            cmd,
             shell=True, check=True)
         click.echo(f"kubectl binary for {architecture} downloaded successfully.")
 
