@@ -1,20 +1,26 @@
-from typing import Optional, List
+import pprint
+from typing import Optional, List, Dict
 
 import click
 import inquirer
 from rich.console import Console
 from rich.table import Table
 from rich.tree import Tree
-from pprint import pprint
 
 from tensorkube.constants import DEFAULT_NAMESPACE, DATASET_BUCKET_TYPE
 from tensorkube.services.aws_service import get_bucket_name
 from tensorkube.services.dataset_service import list_datasets
-from tensorkube.services.k8s_service import start_streaming_pod, ssh_into_pod_with_podman, list_jobs, get_pods_for_jobs, \
-    delete_job
+from tensorkube.services.k8s_service import start_streaming_pod, ssh_into_pod, list_jobs, get_pods_for_jobs, \
+    delete_job, ssh_into_pod_with_podman
 from tensorkube.services.knative_service import list_deployed_services, get_knative_service, get_ready_condition, \
-    get_pods_for_service, get_istio_ingress_gateway_hostname
-from tensorkube.services.train import get_training_id_from_job_name
+    get_pods_for_service, get_istio_ingress_gateway_hostname, delete_knative_service_by_name
+from tensorkube.services.tls_service import get_all_virtualservices, find_domains_for_service, is_custom_domain
+from tensorkube.services.train import get_training_id_from_job_name, get_training_stats
+
+# Message Constants
+
+DELETION_CONFIRMATION_MESSAGE = "Are you sure you want to delete this deployment? This action cannot be undone."
+DELETION_SECOND_CONFIRMATION_WITH_TYPING = "You have chosen to delete the deployment. Please type the service name to be deleted to confirm."
 
 
 def list_tensorkube_deployments(env_name: Optional[str] = None, all: bool = False, old: bool = False):
@@ -32,9 +38,11 @@ def list_tensorkube_deployments(env_name: Optional[str] = None, all: bool = Fals
     if not deployed_services:
         return
     services = deployed_services['items']
+    namespaces = env_name if env_name else DEFAULT_NAMESPACE
+    virtual_services = get_all_virtualservices(namespace=namespaces)
     for service in services:
-
         ready_condition = get_ready_condition(service)
+        service_name = service['metadata']['name']
         if 'latestReadyRevisionName' in service['status']:
             latest_ready_revision = service['status']['latestReadyRevisionName'][-4:]
         else:
@@ -42,15 +50,19 @@ def list_tensorkube_deployments(env_name: Optional[str] = None, all: bool = Fals
         if old:
             service_url = service['status']['url']
         else:
-            service_url = f'http://{elb_url}/svc/{service["metadata"]["namespace"]}/{service["metadata"]["name"]}/'
-        service_name = service['metadata']['name']
+            domains = find_domains_for_service(virtualservices=virtual_services,service_name=service_name)
+            custom_domains = list(filter(is_custom_domain, domains.domains))
+            if custom_domains:
+                service_url = f'https://{custom_domains[0]}/'
+            else:
+                service_url = f'http://{elb_url}/svc/{service["metadata"]["namespace"]}/{service["metadata"]["name"]}/'
         service_env = service['metadata']['namespace']
         tree = Tree("[bold][bright_magenta]" + service_name)
         tree.add("[bold]env:[/] " + service_env)
         tree.add("[bold]URL:[/] [cyan]" + service_url)
         trees.append(tree)
-        table.add_row(service_name, latest_ready_revision, ready_condition['status'],
-                      service_env, ready_condition.get('reason', None))
+        table.add_row(service_name, latest_ready_revision, ready_condition['status'], service_env,
+                      ready_condition.get('reason', None))
 
     console = Console()
     console.print(table)
@@ -83,6 +95,27 @@ def describe_deployment(service_name: str, env_name: Optional[str] = None):
     console = Console()
     console.print(tree)
 
+def delete_deployment(service_name: str, env_name:str):
+    # First ask the user to confirm the deletion
+    # Then ask them to type the service name in addition to a warning that this action is irreversible
+    deployment = get_knative_service(service_name=service_name, namespace=env_name)
+    if deployment is None:
+        click.echo(f"Service {service_name} not found in environment {env_name}")
+        return
+    deletion_confirmation_message = DELETION_CONFIRMATION_MESSAGE
+    if not click.confirm(click.style(deletion_confirmation_message, fg='red')):
+        click.echo("Deletion cancelled.")
+        return
+    deletion_second_confirmation_with_typing = DELETION_SECOND_CONFIRMATION_WITH_TYPING
+    # Now ask it to type the service name and if it matches then only delete
+    if click.prompt(click.style(deletion_second_confirmation_with_typing, fg='red')) == service_name:
+        click.echo("Deleting service... This process will exit once the service is deleted.")
+        # TODO this only deletes the service - The repository and other resources are not deleted
+        delete_knative_service_by_name(service_name=service_name, namespace=env_name, wait=True)
+        click.echo(f"Service {service_name} deleted.")
+    else:
+        click.echo("Service name match failed. Deletion cancelled.")
+
 
 def display_deployment_logs(service_name: str, namespace: str = "default"):
     services_in_namespaces = list_deployed_services(env_name=namespace)['items']
@@ -92,7 +125,8 @@ def display_deployment_logs(service_name: str, namespace: str = "default"):
         return
     service_pods = get_pods_for_service(service_name=service_name, namespace=namespace)
     if service_pods is None:
-        click.echo(f"Your service failed to initialise. No containers could be started. Check dockerfile or view deployment logs")
+        click.echo(
+            f"Your service failed to initialise. No containers could be started. Check dockerfile or view deployment logs")
         return
     if len(service_pods.items) == 0:
         click.echo(f"No active pods found for service {service_name}.")
@@ -116,7 +150,8 @@ def ssh_into_deployed_service(service_name: str, namespace: str = "default"):
         return
     service_pods = get_pods_for_service(service_name=service_name, namespace=namespace)
     if service_pods is None:
-        click.echo(f"Your service failed to initialise. No containers could be started. Check dockerfile or view deployment logs")
+        click.echo(
+            f"Your service failed to initialise. No containers could be started. Check dockerfile or view deployment logs")
         return
     if len(service_pods.items) == 0:
         click.echo(f"No pods found for service {service_name}")
@@ -128,9 +163,14 @@ def ssh_into_deployed_service(service_name: str, namespace: str = "default"):
         questions = [inquirer.List('pod', message="Please select a pod",
                                    choices=[pod.metadata.name for pod in service_pods.items], ), ]
         pod_name = inquirer.prompt(questions)['pod']
-
+    # Check the image of the pod if it is tensorfuse/podman-nvidia:v1 or quay.io/podman/stable use ssh_into_pod_using_podman
+    # else use ssh_into_pod
+    image = service_pods.items[0].spec.containers[0].image
+    if image == "tensorfuse/podman-nvidia:v1" or image == "quay.io/podman/stable":
+        click.echo(f"SSHing into pod: {pod_name}")
+        ssh_into_pod_with_podman(pod_name=pod_name, namespace=namespace)
     click.echo(f"SSHing into pod: {pod_name}")
-    ssh_into_pod_with_podman(pod_name=pod_name, namespace=namespace)
+    ssh_into_pod(pod_name=pod_name, namespace=namespace)
 
 
 def display_secrets(secrets, namespace: str = DEFAULT_NAMESPACE):
@@ -140,7 +180,7 @@ def display_secrets(secrets, namespace: str = DEFAULT_NAMESPACE):
     if namespace != DEFAULT_NAMESPACE:
         tree = Tree("[bold][bright_magenta]Secrets in env: " + namespace)
     else:
-        tree = Tree("[bold][bright_magenta]Secrets")    
+        tree = Tree("[bold][bright_magenta]Secrets")
     for secret in secrets:
         tree.add("[bold]Name:[/] " + secret.metadata.name)
     console = Console()
@@ -180,11 +220,7 @@ def list_tensorkube_datasets():
         dataset_url = f"s3://{bucket_name}/{dataset.key}"
 
         # Add row to main table
-        table.add_row(
-            dataset_id,
-            dataset.last_modified,
-            dataset.size
-        )
+        table.add_row(dataset_id, dataset.last_modified, dataset.size)
 
         # Create detailed tree view for each dataset
         tree = Tree(f"[bold][bright_magenta]{dataset_id}")
@@ -202,13 +238,14 @@ def list_tensorkube_datasets():
         console.print(tree)
 
 
-def display_job_logs(job_prefix: str, namespace: str = "default"):
+def display_job_logs(job_prefix: str, container_name: str = "user-container", namespace: str = "default"):
     """
     Displays logs from a specified job's pods
 
     Args:
         job_prefix: Prefix of the job name
         namespace: Kubernetes namespace where the job exists
+        container_name: Name of the container whose logs to display
     """
     # Get all jobs in the namespace with the prefix
     jobs_in_namespace = list_jobs(namespace=namespace, job_name_prefix=job_prefix)
@@ -219,13 +256,8 @@ def display_job_logs(job_prefix: str, namespace: str = "default"):
     # If multiple jobs found, ask user to select one
     if len(jobs_in_namespace.items) > 1:
         click.echo("Multiple jobs found. Please select a job.")
-        questions = [
-            inquirer.List(
-                'job',
-                message="Please select a job",
-                choices=[job.metadata.name for job in jobs_in_namespace.items],
-            ),
-        ]
+        questions = [inquirer.List('job', message="Please select a job",
+            choices=[job.metadata.name for job in jobs_in_namespace.items], ), ]
         job_name = inquirer.prompt(questions)['job']
     else:
         job_name = jobs_in_namespace.items[0].metadata.name
@@ -236,8 +268,7 @@ def display_job_logs(job_prefix: str, namespace: str = "default"):
     if job_pods is None:
         click.echo(click.style(
             f"Your job failed to initialize. No containers could be started. Check configuration or view deployment logs",
-            fg='red'
-        ))
+            fg='red'))
         return
 
     if len(job_pods.items) == 0:
@@ -247,21 +278,12 @@ def display_job_logs(job_prefix: str, namespace: str = "default"):
         pod_name = job_pods.items[0].metadata.name
     else:
         click.echo("Multiple pods found for job. Please select a pod.")
-        questions = [
-            inquirer.List(
-                'pod',
-                message="Please select a pod",
-                choices=[pod.metadata.name for pod in job_pods.items],
-            ),
-        ]
+        questions = [inquirer.List('pod', message="Please select a pod",
+            choices=[pod.metadata.name for pod in job_pods.items], ), ]
         pod_name = inquirer.prompt(questions)['pod']
 
     # Stream logs from the selected pod
-    start_streaming_pod(
-        pod_name=pod_name,
-        namespace=namespace,
-        container_name="user-container"
-    )
+    start_streaming_pod(pod_name=pod_name, namespace=namespace, container_name=container_name)
 
 
 def list_tensorkube_training_jobs(namespace: Optional[str] = None, all: bool = False, job_prefix: Optional[str] = None):
@@ -303,6 +325,7 @@ def list_tensorkube_training_jobs(namespace: Optional[str] = None, all: bool = F
             job_status = "Succeeded"
         elif status.failed:
             job_status = f"Failed ({status.failed} attempts)"
+            completion_time = "N/A"
         elif status.active:
             job_status = "Active"
         else:
@@ -327,13 +350,7 @@ def list_tensorkube_training_jobs(namespace: Optional[str] = None, all: bool = F
         trees.append(tree)
 
         # Add row to main table
-        table.add_row(
-            get_training_id_from_job_name(job_name),
-            job_status,
-            start_time,
-            completion_time,
-            job_namespace
-        )
+        table.add_row(get_training_id_from_job_name(job_name), job_status, start_time, completion_time, job_namespace)
 
     # Print both table and trees
     console = Console()
@@ -343,6 +360,53 @@ def list_tensorkube_training_jobs(namespace: Optional[str] = None, all: bool = F
         console.print(tree)
 
 
+def get_tensorkube_training_job(job_prefix: str, namespace: str = "default"):
+    """
+    Get job details
+
+    Args:
+        job_prefix: Prefix of the job name
+        namespace: Kubernetes namespace where the job exists
+    """
+    # Get all jobs in the namespace with the prefix
+    jobs_in_namespace = list_jobs(namespace=namespace, job_name_prefix=job_prefix)
+    if not jobs_in_namespace or len(jobs_in_namespace.items) == 0:
+        click.echo(click.style(f"No jobs with prefix '{job_prefix}' found in environment {namespace}", fg='red'))
+        return
+
+    # TODO: ideally there should be a single job with the prefix
+    job = jobs_in_namespace.items[0]
+    status = job.status
+    start_time = status.start_time.strftime("%Y-%m-%d %H:%M:%S") if status.start_time else "N/A"
+    completion_time = status.completion_time.strftime("%Y-%m-%d %H:%M:%S") if status.completion_time else "Running"
+
+    # Determine job status
+    if status.succeeded:
+        job_status = "Succeeded"
+    elif status.failed:
+        job_status = f"Failed ({status.failed} attempts)"
+        completion_time = "N/A"
+    elif status.active:
+        job_status = "Active"
+    else:
+        job_status = "Pending"
+    stats = ''
+    if job_status == "Succeeded" or job_status == "Active":
+        # Convert start_time to milliseconds
+        start_time = int(status.start_time.timestamp())
+        stats = get_training_stats(job.metadata.name, start_time, namespace)
+    table = Table(show_header=False, show_lines=True)
+    table.add_column("Field", style="cyan", no_wrap=True)
+    table.add_column("Value", style="green")
+
+    # Add rows to the table
+    table.add_row("Status", job_status)
+    table.add_row("Completion Time", completion_time)
+    table.add_row("Info", str(stats))
+    console = Console()
+    console.print(table)
+
+
 def delete_tensorkube_job_by_prefix(job_prefix: str, namespace: str = "default") -> bool:
     """
     Deletes a job matching the given prefix. If multiple jobs match, asks user to select one.
@@ -363,13 +427,8 @@ def delete_tensorkube_job_by_prefix(job_prefix: str, namespace: str = "default")
     # If multiple jobs found, ask user to select one
     if len(jobs_in_namespace.items) > 1:
         click.echo("Multiple jobs found. Please select a job to delete.")
-        questions = [
-            inquirer.List(
-                'job',
-                message="Please select a job to delete",
-                choices=[job.metadata.name for job in jobs_in_namespace.items],
-            ),
-        ]
+        questions = [inquirer.List('job', message="Please select a job to delete",
+            choices=[job.metadata.name for job in jobs_in_namespace.items], ), ]
         job_name = inquirer.prompt(questions)['job']
     else:
         job_name = jobs_in_namespace.items[0].metadata.name
@@ -396,4 +455,92 @@ def display_keda_scaled_jobs(jobs):
         table.add_row(job['metadata']['name'], job['status']['conditions'][0]['status'])
 
     console = Console()
-    console.print(table)
\ No newline at end of file
+    console.print(table)
+
+
+def prettify_aws_user_keys(aws_access_key_id, aws_secret_access_key):
+    table = Table(title="AWS User Keys")
+    table.add_column("Key", style="magenta", no_wrap=True)
+    table.add_column("Value", style="green", no_wrap=False, overflow="fold")
+    table.add_row("Access Key ID", aws_access_key_id)
+    table.add_row("Secret Access Key", aws_secret_access_key)
+
+    console = Console()
+    console.print(table)
+
+
+def display_queued_jobs(keda_jobs: Dict, dynamodb_jobs: Dict):
+    jobs = keda_jobs['items']
+    if not jobs:
+        click.echo(f"No queued jobs found")
+        return
+    table = Table(title="Queued Jobs", show_lines=True)
+    table.add_column("Job Name", style="magenta", no_wrap=True)
+    table.add_column("Ready", style="green", no_wrap=False, overflow="ellipsis")
+    table.add_column("Job IDs", style="cyan", no_wrap=True)
+    table.add_column("Status", style="green", no_wrap=False, overflow="fold")
+
+    for job in jobs:
+        job_name = job['metadata']['name']
+        ready_status = job['status']['conditions'][0]['status']
+        if dynamodb_jobs.get(job_name, []):
+            job_ids = "\n".join([x['job_id'] for x in dynamodb_jobs[job_name]])
+            job_statuses = "\n".join([x['status'] for x in dynamodb_jobs[job_name]])
+        else:
+            job_ids = "-"
+            job_statuses = "-"
+        table.add_row(job_name, ready_status, job_ids, job_statuses)
+
+    console = Console()
+    console.print(table)
+
+
+def display_specific_queued_job(keda_job: Dict, dynamodb_jobs: List[Dict]):
+    job_name = keda_job['metadata']['name']
+    ready_status = keda_job['status']['conditions'][0]['status']
+
+    table = Table(title=f"Queued Job: {job_name}")
+    table.add_column("Job IDs", style="cyan", no_wrap=True)
+    table.add_column("Status", style="green", no_wrap=False, overflow="ellipsis")
+
+    if dynamodb_jobs:
+        for job in dynamodb_jobs:
+            table.add_row(job['job_id'], job['status'])
+    else:
+        table.add_row("-", "-")
+
+    console = Console()
+    console.print(table)
+    console.print(f"Ready Status: {ready_status}")
+
+
+def display_dns_records_table(domain: str, validation_records: dict):
+    """
+    Display DNS records in a Rich formatted table
+    Args:
+        domain: Domain name
+        validation_records: Dictionary containing validation records
+    """
+    # Get ELB hostname
+    elb_hostname = get_istio_ingress_gateway_hostname()
+    if not elb_hostname:
+        console = Console()
+        console.print("[red]Error: Could not get Istio ingress gateway hostname[/red]")
+        return
+
+    table = Table(title="Required DNS Records")
+    table.add_column("Type", style="cyan", no_wrap=True)
+    table.add_column("Name", style="green")
+    table.add_column("Value", style="magenta")
+
+    # Add validation record
+    if validation_records:
+        v_record = validation_records['validation_record']
+        table.add_row(v_record['type'], v_record['name'], v_record['value'])
+
+    # Add routing record with actual ELB hostname
+    table.add_row("CNAME", f"*.{domain}", elb_hostname)
+
+    console = Console()
+    console.print(table)
+    console.print("\n[yellow]Please configure these records in your DNS provider.[/yellow]")
