@@ -9,7 +9,7 @@ def human_readable_size(size_bytes):
     i = int(math.floor(math.log(size_bytes, 1024)))
     p = math.pow(1024, i)
     s = round(size_bytes / p, 2)
-    return f'{s} {size_names[i]}'[1]
+    return f'{s} {size_names[i]}'
 
 def human_readable_date(dt):
     now = datetime.now(dt.tzinfo)
